//*************************************************************************
//* -----------
//* CDCAnalysis
//* -----------
//* (Description)
//* 	Clasess to analyze CDC data.
//* (Update Record)
//*	2000/05/16  K.Fujii	Original version.
//*************************************************************************

#include "JSFSteer.h"
#include "CDCAnalysis.h"
#include <iostream.h>

ClassImp(CDCAnalysis)

const Int_t CDCAnalysis::kZoneX   = 4;
const Int_t CDCAnalysis::kZoneY   = 3;
const Int_t CDCAnalysis::kNh = kZoneX*kZoneY;

//___________________________________________________________________________
CDCAnalysis::CDCAnalysis(const Char_t *name, const Char_t *title)
	: JSFModule(name, title), fMode(0), fNwires(5), 
	  xChi2(5.), xNsigmas(3.), fTk1st(0), fTklast(0)
{
   hdefr = new THdef2[kNh];
   hdefs = new THdef2[kNh];
   hdefp = new THdef2[kNh];
   hdefz = new THdef2[kNh];
   hdefe = new THdef1[kNh];
   hdefw = new THdef2[kNh];
   hdefg = new THdef2[kNh];
   hdeft = new THdef1[kNh];
   hdefu = new THdef2[kNh];
   hdefa = new THdef2[kNh];
   Char_t st[256], std[256];
   Int_t i;
   for (i = 0; i < kNh; i++) {
      sprintf(st,"CDCAnalysis.hRes%2.2d",i);
      sprintf(std,"Res%2.2d 50,0.,5.,200,-0.2,0.2",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefr[i].title,
			&hdefr[i].nxbin, &hdefr[i].xmin, &hdefr[i].xmax,
			&hdefr[i].nybin, &hdefr[i].ymin, &hdefr[i].ymax);
      sprintf(st,"CDCAnalysis.hPh%2.2d",i);
      sprintf(std,"Ph%2.2d 50,0.,5.,200,0.,10000.",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefs[i].title,
			&hdefs[i].nxbin, &hdefs[i].xmin, &hdefs[i].xmax,
			&hdefs[i].nybin, &hdefs[i].ymin, &hdefs[i].ymax);
      sprintf(st,"CDCAnalysis.hPhPk%2.2d",i);
      sprintf(std,"PhPk%2.2d 50,0.,5.,256,0.,256.",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefp[i].title,
			&hdefp[i].nxbin, &hdefp[i].xmin, &hdefp[i].xmax,
			&hdefp[i].nybin, &hdefp[i].ymin, &hdefp[i].ymax);
      sprintf(st,"CDCAnalysis.hPed%2.2d",i);
      sprintf(std,"Ped%2.2d 50,0.,5.,256,0.,256.",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefz[i].title,
			&hdefz[i].nxbin, &hdefz[i].xmin, &hdefz[i].xmax,
			&hdefz[i].nybin, &hdefz[i].ymin, &hdefz[i].ymax);
      sprintf(st,"CDCAnalysis.hEff%2.2d",i);
      sprintf(std,"Eff%2.2d 10,0.,5.",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g",
      			 hdefe[i].title,
	                &hdefe[i].nxbin, &hdefe[i].xmin, &hdefe[i].xmax);
      sprintf(st,"CDCAnalysis.hTwk%2.2d",i);
      sprintf(std,"Twk%2.2d 256,0.,256.,200,-0.2,0.2",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefw[i].title,
			&hdefw[i].nxbin, &hdefw[i].xmin, &hdefw[i].xmax,
			&hdefw[i].nybin, &hdefw[i].ymin, &hdefw[i].ymax);
      sprintf(st,"CDCAnalysis.hSgt2%2.2d",i);
      sprintf(std,"Sgt2%2.2d 50,0.,5.,100,0.,0.0001",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefg[i].title,
			&hdefg[i].nxbin, &hdefg[i].xmin, &hdefg[i].xmax,
			&hdefg[i].nybin, &hdefg[i].ymin, &hdefg[i].ymax);
      sprintf(st,"CDCAnalysis.hEff2tk%2.2d",i);
      sprintf(std,"Eff2tk%2.2d 50,0.,5.",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g",
      			 hdeft[i].title,
	                &hdeft[i].nxbin, &hdeft[i].xmin, &hdeft[i].xmax);
      sprintf(st,"CDCAnalysis.hSepRes%2.2d",i);
      sprintf(std,"SepRes%2.2d 50,0.,5.,200,-0.2,0.2",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefu[i].title,
			&hdefu[i].nxbin, &hdefu[i].xmin, &hdefu[i].xmax,
			&hdefu[i].nybin, &hdefu[i].ymin, &hdefu[i].ymax);
      sprintf(st,"CDCAnalysis.hTC2Sep%2.2d",i);
      sprintf(std,"TC2Sep%2.2d 128,0.,2048.,50,0.0,5.0",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefa[i].title,
			&hdefa[i].nxbin, &hdefa[i].xmin, &hdefa[i].xmax,
			&hdefa[i].nybin, &hdefa[i].ymin, &hdefa[i].ymax);
   }
   hRes    = new TH2F* [kNh];
   hPhSum  = new TH2F* [kNh];
   hPhPk   = new TH2F* [kNh];
   hPed    = new TH2F* [kNh];
   hEff    = new TH1E* [kNh];
   hTwk    = new TH2F* [kNh];
   hSgt2   = new TH2F* [kNh];
   hEff2tk = new TH1E* [kNh];
   hSepRes = new TH2F* [kNh];
   hTC2Sep = new TH2F* [kNh];
}

//___________________________________________________________________________
CDCAnalysis::~CDCAnalysis()
{
   delete [] hdefr;
   delete [] hdefs;
   delete [] hdefp;
   delete [] hdefz;
   delete [] hdefe;
   delete [] hdefw;
   delete [] hdefg;
   delete [] hdeft;
   delete [] hdefu;
   delete [] hdefa;
   Int_t i;
   for (i = 0; i < kNh; i++) {
   	delete hRes   [i];
   	delete hPhSum [i];
   	delete hPhPk  [i];
   	delete hPed   [i];
	delete hEff   [i];
	delete hTwk   [i];
	delete hSgt2  [i];
	delete hEff2tk[i];
	delete hSepRes[i];
	delete hTC2Sep[i];
   }
   delete [] hRes;
   delete [] hPhSum;
   delete [] hPhPk;
   delete [] hPed;
   delete [] hEff;
   delete [] hTwk;
   delete [] hSgt2;
   delete [] hEff2tk;
   delete [] hSepRes;
   delete [] hTC2Sep;
}

//___________________________________________________________________________
Bool_t CDCAnalysis::Initialize()
{
   TDirectory *last = gDirectory;
   gFile->cd("/");
   Int_t i;
   for (i = 0; i < kNh; i++) {
      hRes[i] = new TH2F(hdefr[i].title,hdefr[i].title,
      			hdefr[i].nxbin,hdefr[i].xmin,hdefr[i].xmax,
      			hdefr[i].nybin,hdefr[i].ymin,hdefr[i].ymax);
      cout << i << " "      << hdefr[i].title 
      		<< " nx "   << hdefr[i].nxbin
      		<< " xmin " << hdefr[i].xmin
      		<< " xmax " << hdefr[i].xmax
      		<< " ny "   << hdefr[i].nybin
      		<< " ymin " << hdefr[i].ymin
      		<< " ymax " << hdefr[i].ymax
      		<< endl;
   }
   for (i = 0; i < kNh; i++) {
      hPhSum[i] = new TH2F(hdefs[i].title,hdefs[i].title,
      			hdefs[i].nxbin,hdefs[i].xmin,hdefs[i].xmax,
      			hdefs[i].nybin,hdefs[i].ymin,hdefs[i].ymax);
      cout << i << " "      << hdefs[i].title 
      		<< " nx "   << hdefs[i].nxbin
      		<< " xmin " << hdefs[i].xmin
      		<< " xmax " << hdefs[i].xmax
      		<< " ny "   << hdefs[i].nybin
      		<< " ymin " << hdefs[i].ymin
      		<< " ymax " << hdefs[i].ymax
      		<< endl;
   }
   for (i = 0; i < kNh; i++) {
      hPhPk[i] = new TH2F(hdefp[i].title,hdefp[i].title,
      			hdefp[i].nxbin,hdefp[i].xmin,hdefp[i].xmax,
      			hdefp[i].nybin,hdefp[i].ymin,hdefp[i].ymax);
      cout << i << " "      << hdefp[i].title 
      		<< " nx "   << hdefp[i].nxbin
      		<< " xmin " << hdefp[i].xmin
      		<< " xmax " << hdefp[i].xmax
      		<< " ny "   << hdefp[i].nybin
      		<< " ymin " << hdefp[i].ymin
      		<< " ymax " << hdefp[i].ymax
      		<< endl;
   }
   for (i = 0; i < kNh; i++) {
      hPed[i] = new TH2F(hdefz[i].title,hdefz[i].title,
      			hdefz[i].nxbin,hdefz[i].xmin,hdefz[i].xmax,
      			hdefz[i].nybin,hdefz[i].ymin,hdefz[i].ymax);
      cout << i << " "      << hdefz[i].title 
      		<< " nx "   << hdefz[i].nxbin
      		<< " xmin " << hdefz[i].xmin
      		<< " xmax " << hdefz[i].xmax
      		<< " ny "   << hdefz[i].nybin
      		<< " ymin " << hdefz[i].ymin
      		<< " ymax " << hdefz[i].ymax
      		<< endl;
   }
   for (i = 0; i < kNh; i++) {
      hEff[i] = new TH1E(hdefe[i].title,hdefe[i].title,
      			hdefe[i].nxbin,hdefe[i].xmin,hdefe[i].xmax);
      cout << i << " "      << hdefe[i].title 
      		<< " nx "   << hdefe[i].nxbin
      		<< " xmin " << hdefe[i].xmin
      		<< " xmax " << hdefe[i].xmax
      		<< endl;
   }
   for (i = 0; i < kNh; i++) {
      hTwk[i] = new TH2F(hdefw[i].title,hdefw[i].title,
      			hdefw[i].nxbin,hdefw[i].xmin,hdefw[i].xmax,
      			hdefw[i].nybin,hdefw[i].ymin,hdefw[i].ymax);
      cout << i << " "      << hdefw[i].title 
      		<< " nx "   << hdefw[i].nxbin
      		<< " xmin " << hdefw[i].xmin
      		<< " xmax " << hdefw[i].xmax
      		<< " ny "   << hdefw[i].nybin
      		<< " ymin " << hdefw[i].ymin
      		<< " ymax " << hdefw[i].ymax
      		<< endl;
   }
   for (i = 0; i < kNh; i++) {
      hSgt2[i] = new TH2F(hdefg[i].title,hdefg[i].title,
      			hdefg[i].nxbin,hdefg[i].xmin,hdefg[i].xmax,
      			hdefg[i].nybin,hdefg[i].ymin,hdefg[i].ymax);
      cout << i << " "      << hdefg[i].title 
      		<< " nx "   << hdefg[i].nxbin
      		<< " xmin " << hdefg[i].xmin
      		<< " xmax " << hdefg[i].xmax
      		<< " ny "   << hdefg[i].nybin
      		<< " ymin " << hdefg[i].ymin
      		<< " ymax " << hdefg[i].ymax
      		<< endl;
   }
   for (i = 0; i < kNh; i++) {
      hEff2tk[i] = new TH1E(hdeft[i].title,hdeft[i].title,
      			hdeft[i].nxbin,hdeft[i].xmin,hdeft[i].xmax);
      cout << i << " "      << hdeft[i].title 
      		<< " nx "   << hdeft[i].nxbin
      		<< " xmin " << hdeft[i].xmin
      		<< " xmax " << hdeft[i].xmax
      		<< endl;
   }
   for (i = 0; i < kNh; i++) {
      hSepRes[i] = new TH2F(hdefu[i].title,hdefu[i].title,
      			hdefu[i].nxbin,hdefu[i].xmin,hdefu[i].xmax,
      			hdefu[i].nybin,hdefu[i].ymin,hdefu[i].ymax);
      cout << i << " "      << hdefu[i].title 
      		<< " nx "   << hdefu[i].nxbin
      		<< " xmin " << hdefu[i].xmin
      		<< " xmax " << hdefu[i].xmax
      		<< " ny "   << hdefu[i].nybin
      		<< " ymin " << hdefu[i].ymin
      		<< " ymax " << hdefu[i].ymax
      		<< endl;
   }
   for (i = 0; i < kNh; i++) {
      hTC2Sep[i] = new TH2F(hdefa[i].title,hdefa[i].title,
      			hdefa[i].nxbin,hdefa[i].xmin,hdefa[i].xmax,
      			hdefa[i].nybin,hdefa[i].ymin,hdefa[i].ymax);
      cout << i << " "      << hdefa[i].title 
      		<< " nx "   << hdefa[i].nxbin
      		<< " xmin " << hdefa[i].xmin
      		<< " xmax " << hdefa[i].xmax
      		<< " ny "   << hdefa[i].nybin
      		<< " ymin " << hdefa[i].ymin
      		<< " ymax " << hdefa[i].ymax
      		<< endl;
   }
   hcp = new TCanvas("hcp","Canvas p",20, 20, 200*kZoneX, 200*kZoneY);
   hcp->Divide(kZoneX,kZoneY);
   hcs = new TCanvas("hcs","Canvas s",20, 20, 200*kZoneX, 200*kZoneY);
   hcs->Divide(kZoneX,kZoneY);
   hcz = new TCanvas("hcz","Canvas z",20, 20, 200*kZoneX, 200*kZoneY);
   hcz->Divide(kZoneX,kZoneY);
   hcr = new TCanvas("hcr","Canvas r",20, 20, 200*kZoneX, 200*kZoneY);
   hcr->Divide(kZoneX,kZoneY);
   hce = new TCanvas("hce","Canvas e",20, 20, 200*kZoneX, 200*kZoneY);
   hce->Divide(kZoneX,kZoneY);
   hcw = new TCanvas("hcw","Canvas w",20, 20, 200*kZoneX, 200*kZoneY);
   hcw->Divide(kZoneX,kZoneY);
   hcg = new TCanvas("hcg","Canvas g",20, 20, 200*kZoneX, 200*kZoneY);
   hcg->Divide(kZoneX,kZoneY);
   hct = new TCanvas("hct","Canvas t",20, 20, 200*kZoneX, 200*kZoneY);
   hct->Divide(kZoneX,kZoneY);
   hcu = new TCanvas("hcu","Canvas u",20, 20, 200*kZoneX, 200*kZoneY);
   hcu->Divide(kZoneX,kZoneY);
   hca = new TCanvas("hca","Canvas a",20, 20, 200*kZoneX, 200*kZoneY);
   hca->Divide(kZoneX,kZoneY);
   last->cd();
   return kTRUE;
}


//___________________________________________________________________________
Bool_t CDCAnalysis::Process(Int_t ev)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");

   CDCTrackMaker *tm = (CDCTrackMaker*)gJSF->FindModule("CDCTrackMaker");
   CDCTrackBuf   *tb = (CDCTrackBuf *)tm->EventBuf();
   TObjArray     *ta = tb->GetTracks();

   CDCTrack *tp;
   CDCTrack *tplst = 0;
   TIter nexttrack(ta);
   TObjArray tracks[2];
   while ((tp = (CDCTrack *)nexttrack())) {
      CDCHit *htp = ((CDCTrackHit *)tp->At(0))->GetCDCHit();
      Int_t thislayer = htp->GetLayerNo();
      Int_t thiscell  = htp->GetCellNo();
      if (thiscell != 0) continue;		// skip unequipped cell
      tracks[thislayer].Add(tp);
   }

   Int_t itk1st  = fTk1st;			// track No. range (1st)
   Int_t itklst  = fTklast;			//		   (lst)

   Double_t adctc1 = 0.;
   Double_t adctc2 = 0.;
   Double_t adccsi = 0.;
   GetCounterInfo(adctc1,adctc2,adccsi);

   for (Int_t ilyr = 0; ilyr < 2; ilyr++) {   
      Int_t ntracks = tracks[ilyr].GetEntries();
      itklst = TMath::Min(fTklast,ntracks-1);
      for (Int_t itk = itk1st; itk <= itklst; itk++) {
         tp = (CDCTrack *)tracks[ilyr].At(itk);
         if (!tplst && itk > 0) tplst = (CDCTrack *)tracks[ilyr].At(itk-1);
         Int_t nwires = tp->GetEntries();
         if (nwires < fNwires) continue;	// require fNwires wires
         CDCTrackHit *hitp;
         CDCHit      *htp;
         CDCTrack     trk(*tp);
         TIter next(&trk);
         Stat_t wat[CDCAnalysis::kNh];
         for (Int_t i = 0; i < CDCAnalysis::kNh; i++) wat[i] = 0;
         Int_t tolook;
         for (tolook = 5*ilyr; tolook < 5*(ilyr+1); tolook++) {
      	    next.Reset();
      	    Int_t nref = 0;
      	    CDCTrackHit *actualp = 0;
            while ((hitp = (CDCTrackHit *)next())) {
               htp = hitp->GetCDCHit();
               Int_t layer = htp->GetLayerNo();
               Int_t cell  = htp->GetCellNo();
               Int_t wire  = htp->GetWireNo();
               Int_t a     = 5*layer + wire;	// works only for baby
               if (cell == 1) a = 10 + wire;	// works only for baby
               if ( a < 0 || a >= CDCAnalysis::kNh ) continue;
               if (a == tolook) {
                  actualp = hitp;
               } else { 
                  nref++;
               }
            }
            if (nref < 4) continue;
            if (actualp && fMode == 0) {
	       actualp->Lock();			// lock this hit and
               trk.Update();			// exclude it from fit
               if (trk.GetChi2() > xChi2) {
                  actualp->Unlock();
                  break;
               }
            }
            if (actualp) {
               htp = actualp->GetCDCHit();
               Int_t layer = htp->GetLayerNo();
               Int_t cell  = htp->GetCellNo();
               Int_t wire  = htp->GetWireNo();
               Int_t a     = 5*layer + wire;	// works only for baby
               if (cell == 1) a = 10 + wire;	// works only for baby
               Double_t y    = htp->Ywire();
               Double_t x    = trk.Xat(y) - htp->Xwire();
	       if (layer == 1 && cell == 0) x *= -1.;
               Double_t dx   = actualp->GetResidual();
               Double_t ph   = actualp->GetCDCHit()->GetCluster()->GetSum();
               Double_t php  = actualp->GetCDCHit()->GetCluster()->GetPeak();
               Double_t ped  = actualp->GetCDCHit()->GetCluster()->GetPedestal();
               Double_t sgx2 = trk.SGX2at(htp->Ywire());
	       // cerr << "sgx2 = " << sgx2 << endl;
      	       hRes  [a]->Fill(x,dx,1.0);
      	       hPhSum[a]->Fill(x,ph,1.0);
      	       hPhPk [a]->Fill(x,php,1.0);
      	       hPed  [a]->Fill(x,ped,1.0);
               hTwk  [a]->Fill(php,dx,1.0);
               hSgt2 [a]->Fill(x,sgx2,1.0);
               if (tplst) {
                  Double_t sep = tp->Xat(y) - tplst->Xat(y);
	          if (layer == 1 && cell == 0) sep *= -1.;
                  hSepRes[a]->Fill(sep,dx,1.0);
                  hTC2Sep[a]->Fill(adctc2,sep,1.0);
               }
               if (TMath::Abs(dx) < xNsigmas*TMath::Sqrt(sgx2)) wat[a] = 1.;
               if (fMode == 0) {
	          actualp->Unlock();		// unlock this hit
	       }
            }
      	    Double_t x, y;
      	    Int_t layer, cell, wire;
      	    CDCHitWire::ChannelToWire(tolook,layer,cell,wire,x,y);
      	    hEff[tolook]->Fill(tp->Xat(y),wat[tolook]);
      	    if (tplst) {
      	       Double_t sep = tp->Xat(y) - tplst->Xat(y);
	       if (layer == 1 && cell == 0) sep *= -1.;
      	       hEff2tk[tolook]->Fill(sep,wat[tolook]);
      	    }
         }
         tplst = tp;
      }
      if (!(gROOT->IsBatch()) && ev%200 == 1) Draw();
   }
   last->cd();
   return kTRUE;
}

//___________________________________________________________________________
void CDCAnalysis::Draw(Option_t *opt)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");
   Int_t i;
   for (i = 0; i < CDCAnalysis::kNh; i++) {
      hcp->cd(i+1); hPhPk  [i]->Draw(); hcp->Update();
      hcs->cd(i+1); hPhSum [i]->Draw(); hcs->Update();
      hcz->cd(i+1); hPed   [i]->Draw(); hcz->Update();
      hcr->cd(i+1); hRes   [i]->Draw(); hcr->Update();
      hce->cd(i+1); hEff   [i]->Draw();	hce->Update();
      hcw->cd(i+1); hTwk   [i]->Draw();	hcw->Update();
      hcg->cd(i+1); hSgt2  [i]->Draw();	hcg->Update();
      hct->cd(i+1); hEff2tk[i]->Draw();	hct->Update();
      hcu->cd(i+1); hSepRes[i]->Draw();	hcu->Update();
      hca->cd(i+1); hTC2Sep[i]->Draw();	hca->Update();
   }
   last->cd();
}

//---------------------------------------------------------------------------
void CDCAnalysis::GetCounterInfo(Double_t &adctc1, Double_t &adctc2, 
                                 Double_t &adccsi)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");
   CDCRawData     *raw = (CDCRawData*)gJSF->FindModule("CDCRawData");
   CDCRawDataBuf  *evt = (CDCRawDataBuf *)raw->EventBuf();
   CDCBeginRunBuf *br  = raw->GetBeginRunBuf();
   Int_t runno = gJSF->GetRunNumber();
   if (runno < 820) return;

   Int_t              ncdc = evt->GetNADCData();
   TClonesArray   *adcdata = evt->GetADCData();
   Int_t i;
   for (i = 0; i < ncdc; i++) {
      CDCRawADCDatum *adcdatum = (CDCRawADCDatum *)adcdata->UncheckedAt(i);
      Int_t st = adcdatum->GetStationNo();
      if (st <= 13) continue;
      if (st == 15 || st == 17 || st == 19 || st > 20) {
         cerr << "????? Error ?????" << endl;
         cerr << " Event = " << gJSF->GetEventNumber() << endl;
         cerr << " st: " << st << endl;
         continue;
      }
      Int_t a;
      Int_t id;
      Int_t cr = 0;               // We have only 1 crate now.
      Int_t ch = adcdatum->GetChannelNo();
      br->EChannelToDChannel(cr,st,ch,id,a);
      if (id == 0) continue;
      Double_t adc  = adcdatum->GetADC();
      switch (a) {
           case 31: {
                adctc1 = adc;
                break;
           }
           case 33: {
                adctc2 = adc;
                break;
           }
           case 35: {
                adccsi = adc;
                break;
           }
           default:
                break;
        }
   } 
   last->cd();
}

