#ifndef __USERANALYSIS__
#define __USERANALYSIS__

#include "BabyDisplay.h"
#include "TStyle.h"
#include "TH1.h"
#include "TH2.h"
#include "TGraph.h"
#include "TMath.h"
#include "JSFSteer.h"
#include "CDCTrackMaker.h"

const Int_t  kNhodos = 32;
const Double_t  kHodThresh = 30.;
const Int_t  kNmods  = 10;
#if 0
const Int_t  kFullDriftTime = 8196;
#else
// const Int_t  kFullDriftTime = 6324;
const Int_t  kFullDriftTime = 6492;
#endif
const Int_t  kT0[2][12] 
	= {{412,412, 410,410, 338,332, 320,320, 226,223, 682,682},
	   {678,678, 676,676, 604,598, 588,588, 618,618, 1056,1056}};

#if 0
const Double_t  kHodPed[] = {110., 110., 103., 107.,   102., 129., 117., 125.,
                             121., 113., 120., 118.,   100.,  99.,  97.,  95.,
                             105.,  82.,  89., 100.,   105.,  98., 104., 136.,
                             109., 109., 111., 102.,   104., 107., 111., 116.};
#else
const Double_t  kHodPed[] = {135., 137., 133., 134.,   134., 134., 142., 143.,
			     140., 137., 144., 145.,   131., 129., 133., 129.,
			     131., 111., 122., 127.,   131., 134., 132., 129.,
			     132., 137., 142., 130.,   139., 135., 139., 134.};
#endif

class UserAnalysis {
public:
	UserAnalysis(BabyDisplay *display);
	virtual ~UserAnalysis();

	void InitRawFADC();
	void DrawRawFADC();
	void InitCounters();
	void DrawCounters();

	void Initialize();
	void DrawHist();

	void DeleteHist(Char_t *);
	void DeleteGraph(Char_t *);

private:
        BabyDisplay *fDisplay;		//!
	TH1F        *hHADC[kNhodos];	//!
	TH1F        *hHod1;		//!
	TH1F        *hHod2;		//!
	TH2F        *hHod12;		//!
	TH2F        *hHodADC12;		//!
	TH2F        *hHod12lo;		//!
	TH2F        *hHod12hi;		//!
	TH2F        *hTC1TC2;		//!
	TH2F        *hMonHod;		//!
	TH2F        *hHod1hs;		//!
	TH2F        *hNHod12;		//!
	TH2F        *hMonCsi;		//!
	TH1F        *hFADC[kNmods];	//!

	ClassDef(UserAnalysis, 1)   // A class to analyze event
};

#endif

