#ifndef __CDCRAWADCDATUM__
#define __CDCRAWADCDATUM__

//*************************************************************************
//* --------------------
//* CDCRawADCDatum Class
//* --------------------
//* (Description)
//* 	Classes for a single FADC datum.
//* (Requires)
//*	TObject
//* (Provides)
//*	CDCRawADCDatum
//* (Update Record)
//*	1999/06/02  K.Fujii	Original version.
//*************************************************************************

#include "TObject.h"
#include "CDCUniversalRecordHeader.h"

// static variables are not declared as constant so as to allow
// different formats in future.

class CDCRawADCDatum : public TObject {
public:
  CDCRawADCDatum();
  CDCRawADCDatum(CDCRawADCDatum & rad);
  CDCRawADCDatum(ADC_t adcword[]);
  virtual ~CDCRawADCDatum();

  static void  SetFormat(CDCUniversalRecordHeader & urh);
  inline void  SetStationNo(Int_t n) { fStationNo = n;    }
  inline void  SetChannelNo(Int_t n) { fChannelNo = n;    }
  inline void  SetADC      (Int_t n) { fADC       = n;    }
  inline void  SetTime     (Int_t n) { fTime      = n;    }
  
  inline Int_t GetStationNo() const  { return fStationNo; }
  inline Int_t GetChannelNo() const  { return fChannelNo; }
  inline Int_t GetADC      () const  { return fADC;       }
  inline Int_t GetTime     () const  { return fTime;      }
  static Int_t GetFormatID ()        { return gFormatID;  }
  static Int_t GetRawLength()        { return gNswperdt;  }
  static Int_t GetNchMax   ()        { return gNchMax;    }

  void Unpack(ADC_t adcword[]);
  void DebugPrint();

private:
  Int_t	   fStationNo;			// Station number
  Int_t    fChannelNo;			// Channel number
  Int_t    fADC;			// ADC value
  Int_t    fTime;			// time clock count
  static   Int_t gFormatID;		//! Format ID
  static   Int_t gNstabits;		//! No. of bits for station No
  static   Int_t gNchabits;		//! No. of bits for channel No
  static   Int_t gNswperdt;		//! No. of short words per channel
  static   Int_t gNchMax;		//! Maximum channel No

  ClassDef(CDCRawADCDatum, 1) // CDC raw data buffer
};

#endif
